package fr.jeremylebrun.ld41.core;

import com.badlogic.gdx.ApplicationAdapter;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator;
import com.badlogic.gdx.graphics.g2d.freetype.FreeTypeFontGenerator.FreeTypeFontParameter;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;

import fr.jeremylebrun.ld41.menus.EndgameMenu;
import fr.jeremylebrun.ld41.menus.GameHud;
import fr.jeremylebrun.ld41.menus.GameMenu;
import fr.jeremylebrun.ld41.menus.Menu;
import fr.jeremylebrun.ld41.menus.TitleMenu;


public class Game extends ApplicationAdapter {
	SpriteBatch gameBatch, menuBatch;
	Texture img;
	
	private GameMenu game;
	private TitleMenu title;
	private EndgameMenu endgame;
	
	private Menu currentMenu;
	
	private int mode = 0;
	
	/**
	 * Initialisation du jeu
	 */
	@Override
	public void create () {
		gameBatch = new SpriteBatch();
		menuBatch = new SpriteBatch();
		Shared.GAME_BATCH = gameBatch;
		Shared.MENU_BATCH = menuBatch;
		Shared.SHAPE_RENDERER = new ShapeRenderer();

		// Generation de la font
		FreeTypeFontGenerator generator = new FreeTypeFontGenerator(Gdx.files.internal("hud/pixelOperator8.ttf"));
		FreeTypeFontParameter parameter = new FreeTypeFontParameter();
		parameter.size = 14;
		parameter.spaceX = 0;
		Shared.FONT = generator.generateFont(parameter);
		Shared.FONT.setColor(Color.WHITE);
		
		FreeTypeFontParameter parameter2 = new FreeTypeFontParameter();
		parameter2.size = 10;
		parameter2.spaceX = 0;
		Shared.FONT_SMALL = generator.generateFont(parameter2);
		Shared.FONT_SMALL.setColor(Color.WHITE);
		
		FreeTypeFontParameter parameter3 = new FreeTypeFontParameter();
		parameter3.size = 18;
		parameter3.spaceX = 0;
		Shared.FONT_BIG = generator.generateFont(parameter3);
		Shared.FONT_BIG.setColor(Color.WHITE);
		generator.dispose();
		
		game = new GameMenu();
		title = new TitleMenu();
		endgame = new EndgameMenu();
		
		goToTitleMenu();
		
	}

	/**
	 * Dessin du jeu
	 */
	@Override
	public void render () {
		 Gdx.gl.glClearColor(1, 0, 0, 1);
	     Gdx.gl.glBlendFunc(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA);
	     Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
		
		currentMenu.update(this);
		currentMenu.render(this);
	}
	
	/**
	 * Envoi vers le menu de jeu
	 */
	public void goToGameMenu() {
		game.init(this);
		currentMenu = game;
	}
	
	/**
	 * Envoi vers l'ecran titre
	 */
	public void goToTitleMenu() {
		title.init(this);
		currentMenu = title;
	}
	/**
	 * Envoi vers l'ecran titre
	 */
	public void goToEndgameMenu(GameHud hud) {
		endgame.init(this);
		endgame.setHud(hud);
		currentMenu = endgame;
	}
}
