package fr.jeremylebrun.ld41.entities;

import java.util.ArrayList;
import java.util.List;

import com.badlogic.gdx.math.Vector2;

import fr.jeremylebrun.ld41.core.Shared;
import fr.jeremylebrun.ld41.menus.GameMenu;

public abstract class Entity {
	protected float x, y;
	protected float width, height;
	protected int level;
	protected boolean destroy = false;

	
	
	protected List<Vector2> lstCollisions;
	protected boolean activ = true;
	

	public Entity(float x, float y, float width, float height, int level) {
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.level = level;
		lstCollisions = new ArrayList<>();

	}
	

	/**
	 * Initialise l'entite
	 * @param game
	 */
	public void init(GameMenu game) {
	}
	

	/**
	 * Dessine l'entite
	 * @param game
	 */
	public void render(GameMenu game) {
		
	}
	
	/**
	 * Dessine l'entite
	 * @param game
	 */
	public void renderDebug(GameMenu game) {
		Shared.SHAPE_RENDERER.setColor(1, 1, 0, 1);
		Shared.SHAPE_RENDERER.rect(x-(width/2), y-(height/2), width,height);
		
		if(!lstCollisions.isEmpty()) {
			Shared.SHAPE_RENDERER.setColor(1, 0.5f, 0, 1);
			for (Vector2 v : lstCollisions) {
				Shared.SHAPE_RENDERER.point(v.x, v.y, 0);
			}
		}
	}

	/**
	 * Met a jour l'entite
	 * @param game
	 */
	public void update(GameMenu game) {

	}
	
	public boolean move(GameMenu game, float dx, float dy, boolean onAir, boolean checkCollision) {
		return this.move(game, dx, dy, onAir, checkCollision,false);
	}
	
	public boolean move(GameMenu game, float dx, float dy, boolean onAir, boolean checkCollision, boolean blockBySpace) {
		float realDx = 0;
		float realDy=0;
		if(checkCollision) {
			if(game.getMap().canMove(this, dx, dy,false,blockBySpace)) {
				realDx = dx;
				realDy = dy;
			}else if(dx != 0 && game.getMap().canMove(this, dx, 0,false, blockBySpace)) {
				realDx = dx;
				realDy = 0;
			}else if(dy != 0 && game.getMap().canMove(this, 0, dy,false, blockBySpace)) {
				realDx = 0;
				realDy = dy;
			}else {
				realDx = 0;
				realDy = 0;
			}
		}else {
			realDx = dx;
			realDy = dy;
		}
		
		if(realDx != 0 || realDy != 0) {
			absoluteMove(game, realDx, realDy);
			return true;
		}else {
			return false;
		}
		
	}
	
	protected void absoluteMove(GameMenu game, float dx, float dy) {
		this.x += dx;
		this.y += dy;
		for (Vector2 v : lstCollisions) {
			v.x += dx;
			v.y += dy;
		}
	}
	
	public float getX() {
		return x;
	}
	public float getY() {
		return y;
	}
	public float getWidth() {
		return width;
	}
	public float getHeight() {
		return height;
	}
	public int getLevel() {
		return level;
	}


	public List<Vector2> getLstCollisions() {
		return lstCollisions;
	}


	public void setLstCollisions(List<Vector2> lstCollisions) {
		this.lstCollisions = lstCollisions;
	}
	
	public boolean isDestroy() {
		return destroy;
	}
	
	public boolean contains(Vector2 v, float dx, float dy) {
		
		if(Math.abs((v.x+dx)-this.getX()) < (width/2) &&  Math.abs((v.y+dy)-this.getY()) < (height/2)) {
			return true;
		}
		return false;
	}
	public boolean contains(float ex, float ey) {
		
		if(Math.abs((ex)-this.getX()) < (width/2) &&  Math.abs((ey)-this.getY()) < (height/2)) {
			return true;
		}
		return false;
	}
	
	public boolean onContact(Entity e) {
		if(e.getLevel()!=level || e.isDestroy()) {
			return false;
		}
		
		if(Math.abs(e.getX()-this.getX()) < (width/2) &&  Math.abs(e.getY()-this.getY()) < (height/2)) {
			return true;
		}
		return false;
	}
	
	public boolean isActiv() {
		return activ;
	}
}
