package fr.jeremylebrun.ld41.entities;


import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;

import fr.jeremylebrun.ld41.core.Ressources;
import fr.jeremylebrun.ld41.core.Shared;
import fr.jeremylebrun.ld41.menus.GameMenu;

public class Zombi extends Entity {

	private Animation<TextureRegion> runAnimation  ;
	private Animation<TextureRegion> idleAnimation ;

	private float statetime = 0f;
	private float rotation = 0;
	private int direction= Shared.DIR_UP;
	private float speed = 1f;

	private State state;
	private Vector2 lastMove;

	private float dieTimer = 0f;


	private Vector2 targetPosition;
	private int cptTarget = 0;
	private int cptStop = 0;
	
	private int life = 3;
	private int hit = 0;
	private int type = 0;
	
	public Zombi(GameMenu game,float x, float y, int level, int type) {
		super(x, y, 32, 32,level);
		lastMove = new Vector2(0,0);
		this.type = type;
		this.init(game);
		this.activ = false;
	}


	/**
	 * Initialise l'entite
	 * @param game
	 */
	public void init(GameMenu game) {
		TextureRegion[][] textureRegion = TextureRegion.split(getRandomZombiTexture(),32,32);

		TextureRegion[] runFrames = new TextureRegion[4];
		TextureRegion[] idleFrames = new TextureRegion[1];

		runFrames[0]  = textureRegion[0][0];
		runFrames[1]  = textureRegion[0][1];
		runFrames[2]  = textureRegion[0][0];
		runFrames[3]  = textureRegion[0][2];

		idleFrames[0] = textureRegion[0][0];

		runAnimation = new Animation<TextureRegion>(0.15f, runFrames);
		runAnimation.setPlayMode(Animation.PlayMode.LOOP);

		idleAnimation = new Animation<TextureRegion>(0.15f, idleFrames);
		idleAnimation.setPlayMode(Animation.PlayMode.LOOP);

		state = State.IDLE;

		lstCollisions.add(new Vector2(x-12, y+12));
		lstCollisions.add(new Vector2(x+12, y+12));
		lstCollisions.add(new Vector2(x-12, y-12));
		lstCollisions.add(new Vector2(x+12, y-12));

		if(type == 0) {
			speed = 0.6f;
			life = 3;
		}else if(type == 1) {
			speed = 0.7f;
			life = 4;
		}else if(type == 2) {
			speed = 1.2f;
			life = 2;
		}
		
		super.init(game);
	}

	/**
	 * Dessine l'entite
	 * @param game
	 */
	public void render(GameMenu game) {
		if(activ && state != State.DIE && dieTimer<=0f && !destroy) {
			TextureRegion currentFrame;
			if(state == State.IDLE) {
				currentFrame = idleAnimation.getKeyFrame(statetime);
			}else if(state == State.MOVE) {
				currentFrame = runAnimation.getKeyFrame(statetime);
			}else {
				currentFrame = idleAnimation.getKeyFrame(statetime);
			}

			if(hit>0) {
				Shared.GAME_BATCH.setColor(1f, 1f-(hit*0.15f), 1f-(hit*0.15f), 1f);
			}
			Shared.GAME_BATCH.draw(currentFrame, (int)(x-width/2),  (int)(y-height/2),16,16,width,height,1,1,rotation);
			super.render(game);

			Shared.GAME_BATCH.setColor(1f, 1f, 1f, 1f);
		}
		

		statetime += Gdx.graphics.getDeltaTime(); 

	}

	/**
	 * Met a jour l'entite
	 * @param game
	 */
	public void update(GameMenu game) {
		if(!destroy ) {

			super.update(game);

			// GESTION DE LA MORT 
			if(dieTimer > 0f) {
				dieTimer -= 0.05f;
				if(dieTimer <=0f) {
					dieTimer = 0f;
					this.destroy = true;
				}
				return;
			}

			if(!game.getPlayer().toFarFromPlayer(this)) {
				if(state == State.IDLE) {
					lastMove.x = 0f;
					lastMove.y = 0f;
				}
				if(!activ) {
					activ = true;
				}

				updateCollisionWithBullet(game);
				
				if(!game.getPlayer().toFarFromPlayerClose(this)) {
					if(cptStop<=0) {
						this.updateDeplacement(game);
					}else {
						cptStop--;
					}
					this.updateCollisionWithPlayer(game);
				}
			

				this.rotation = Shared.getRotationFromDirection(direction);
			}
			
		}
	}
	
	private void updateCollisionWithBullet(GameMenu game) {
		PlayerBullet pb;
		
		for (Entity e : game.getLayerEntities(level)) {
			
			if(e instanceof PlayerBullet) {
				if(!e.isDestroy() && e.isActiv() && this.contains(e.getX(), e.getY())) {
					pb = (PlayerBullet)e;
					pb.destroyByContact(game);
					
					game.getTmpLayerEntities(level).add(new SimpleAnimation((x-8),(y-8),16,16,level,Shared.BLOOD_ANIMATIONS[(int) (Math.random()*8)]));
					
					hitByBullet(game);
				}
			}
		}
	}
	
	private void hitByBullet(GameMenu game) {
		this.hit++;
		this.cptStop = 15;
		this.state = State.IDLE;
		
		Ressources.SOUND_SHOOT_IMPACT.play();
		if(hit>=life) {
			dieTimer = 1f;
			game.getHud().incKills();
			game.getTmpLayerEntities(level).add(new EnemiFlesh(x, y, level, Ressources.TEXTURE_ZOMBI_FLESH[type==1?2:(int)(Math.random()*2)][0], true,true));
			game.getTmpLayerEntities(level).add(new EnemiFlesh(x, y, level, Ressources.TEXTURE_ZOMBI_FLESH[type==1?2:(int)(Math.random()*2)][1], true,true));
			game.getTmpLayerEntities(level).add(new EnemiFlesh(x, y, level, Ressources.TEXTURE_ZOMBI_FLESH[type==1?2:(int)(Math.random()*2)][2], true,true));
			game.getTmpLayerEntities(level).add(new EnemiFlesh(x, y, level, Ressources.TEXTURE_ZOMBI_FLESH[type==1?2:(int)(Math.random()*2)][3], true,true));
		}
	}

	private void updateDeplacement(GameMenu game) {
		if(targetPosition == null || cptTarget<=0) {
			targetPosition = new Vector2(game.getPlayer().getX(),game.getPlayer().getY());
			cptTarget = (int)(Math.random() * 200)+50;
		}

		cptTarget--;
		float dx = 0; 
		float dy = 0;

		float distanceX = Math.abs(targetPosition.x - x);
		float distanceY = Math.abs(targetPosition.y - y);
		
		if(distanceX>8) {
			if(targetPosition.x > x) {
				dx = 1;
			}else if(targetPosition.x < x) {
				dx -= 1;
			}
		}
		
		if(distanceY>8) {
			if(targetPosition.y > y) {
				dy += 1;
			}else if(targetPosition.y < y) {
				dy -= 1;
			}
		}

		dx *= speed;
		dy *=speed;

		if(dx != 0 && dy != 0) {
			dx *= 0.7f;
			dy *=0.7f;
		}

		direction = Shared.getDirectionFromDeplacement(dx, dy);
		if((dx == 0 && dy==0) || !this.move(game, dx, dy, false, true, true) ) {
			state = State.IDLE;
			cptTarget = 0;
		}else {
			state = State.MOVE;
		}
	}

	private void updateCollisionWithPlayer(GameMenu game) {
		if(game.getPlayer().getLevel() == level) {
			for (Vector2 v : game.getPlayer().getLstCollisions()) {
				if(Math.abs(v.x-this.getX()) < 12 &&  Math.abs(v.y-this.getY()) < 12) {
					state = State.IDLE;
					cptStop = 50;
					game.getPlayer().hitByEnemy(game,this);
				}
			}
		}
	}


	@Override
	protected void absoluteMove(GameMenu game, float dx, float dy) {

		lastMove.x = dx;
		lastMove.y = dy;
		super.absoluteMove(game, dx, dy);
	}

	public boolean onContact(Entity e) {
		if(e.getLevel()!=level) {
			return false;
		}

		if(Math.abs(e.getX()-this.getX()) < 12 &&  Math.abs(e.getY()-this.getY()) < 12) {
			return true;
		}
		return false;
	}
	
	private Texture getRandomZombiTexture() {
		if(type == 1) {
			return Ressources.TX_ZOMBI4;
		}else if(type == 2) {
				return Ressources.TX_ZOMBI5;
		}else {
			double r = Math.random();
			if(r < 0.33f) {
				return Ressources.TX_ZOMBI1;
			}else if(r < 0.66f) {
				return Ressources.TX_ZOMBI2;
			}else {
				return Ressources.TX_ZOMBI3;
			}
		}
		
		
	}
}
