package fr.jeremylebrun.ld41.map;

import java.util.ArrayList;
import java.util.List;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.maps.tiled.TiledMap;
import com.badlogic.gdx.maps.tiled.TiledMapRenderer;
import com.badlogic.gdx.maps.tiled.TiledMapTileLayer;
import com.badlogic.gdx.maps.tiled.TiledMapTileLayer.Cell;
import com.badlogic.gdx.maps.tiled.TmxMapLoader;
import com.badlogic.gdx.maps.tiled.renderers.OrthogonalTiledMapRenderer;
import com.badlogic.gdx.math.Vector2;

import fr.jeremylebrun.ld41.core.Shared;
import fr.jeremylebrun.ld41.entities.Entity;
import fr.jeremylebrun.ld41.entities.PlayerBullet;
import fr.jeremylebrun.ld41.menus.GameMenu;

public class GameMap  {
	private TiledMap tiledMap;
	private TiledMapRenderer tiledMapRenderer;
	private int[] layers0;
	private int[] layers1;
	private int[] layers2;
	private int absoluteCollisionLayerIndex;
	
	private List<GameMapCollisionRectangle> lstBlockCollision;
	private List<GameMapCollisionRectangle> lstPassableCollision;
	
	
	public void init (GameMenu game, int levelIndex) {
		float w = Shared.GAME_WIDTH;
		float h =  Shared.GAME_HEIGHT;
		
		lstBlockCollision = new ArrayList<>();
		lstPassableCollision = new ArrayList<>();

		game.camera = new OrthographicCamera();
		game.camera.setToOrtho(false,w,h);
		game.camera.translate(0, 96);
		game.camera.update();
		tiledMap = new TmxMapLoader().load("map/level"+levelIndex+".tmx");
		tiledMapRenderer = new OrthogonalTiledMapRenderer(tiledMap);
		
		layers0 = new int[2];
		layers0[0] = 0;
		layers0[1] = 1;
		
		layers1 = new int[2];
		layers1[0] = 3;
		layers1[1] = 4;
		
		layers2 = new int[2];
		layers2[0] = 6;
		layers2[1] = 7;
		
		absoluteCollisionLayerIndex = 9;
	
	}


	public void update(GameMenu game) {
	}

	public void render(GameMenu game) {
		game.camera.update();
		tiledMapRenderer.setView(game.camera);
	
	}
	
	public boolean canMove(Entity e, float dx, float dy, boolean blockByWallToo, boolean blockBySpace) {
		boolean canMove = true;

		TiledMapTileLayer absoluteCollisionLayer = (TiledMapTileLayer)tiledMap.getLayers().get(absoluteCollisionLayerIndex);
		TiledMapTileLayer currentLayer = (TiledMapTileLayer)tiledMap.getLayers().get(getCurrentMapLayerIndex(e));
		
		Cell absoluteCell;
		Cell currentCell;
		
		
		if(!e.getLstCollisions().isEmpty()) {
			for (Vector2 v : e.getLstCollisions()) {
				absoluteCell = absoluteCollisionLayer.getCell((int)((v.x+dx)/32), (int)((v.y+dy)/32));
				currentCell = currentLayer.getCell((int)((v.x+dx)/32), (int)((v.y+dy)/32));
				
				if(absoluteCell != null && absoluteCell.getTile() != null && "block".equals(absoluteCell.getTile().getProperties().get("collision"))){
					return false;
				}
				
				if(currentCell != null && currentCell.getTile() != null && "block".equals(currentCell.getTile().getProperties().get("collision"))){
					return false;
				}
				
				if(blockByWallToo && absoluteCell != null && absoluteCell.getTile() != null && "wall".equals(absoluteCell.getTile().getProperties().get("collision"))){
					return false;
				}
				
				if(blockByWallToo && currentCell != null && currentCell.getTile() != null && "wall".equals(currentCell.getTile().getProperties().get("collision"))){
					return false;
				}
				
				for (GameMapCollisionRectangle r : lstBlockCollision) {
					if(r.getLevel() == e.getLevel() && !r.isDestroy() && r.contains(v,dx,dy)) {
						if(e instanceof PlayerBullet) {
							r.contactWithBullet();
						}
						return false;
					}
				}
			}
		}else {
			absoluteCell = absoluteCollisionLayer.getCell((int)((e.getX()+dx)/32), (int)((e.getY()+dy)/32));
			currentCell = currentLayer.getCell((int)((e.getX()+dx)/32), (int)((e.getY()+dy)/32));
			
			if(absoluteCell != null && absoluteCell.getTile() != null && "block".equals(absoluteCell.getTile().getProperties().get("collision"))){
				return false;
			}
			
			if(currentCell != null && currentCell.getTile() != null && "block".equals(currentCell.getTile().getProperties().get("collision"))){
				return false;
			}
			
			if(blockByWallToo && absoluteCell != null && absoluteCell.getTile() != null && "wall".equals(absoluteCell.getTile().getProperties().get("collision"))){
				return false;
			}
			
			if(blockByWallToo && currentCell != null && currentCell.getTile() != null && "wall".equals(currentCell.getTile().getProperties().get("collision"))){
				return false;
			}
			
			for (GameMapCollisionRectangle r : lstBlockCollision) {
				if(r.getLevel() == e.getLevel() && !r.isDestroy() && r.contains(new Vector2(e.getX(),e.getY()),dx,dy)) {
					if(e instanceof PlayerBullet) {
						r.contactWithBullet();
					}
					return false;
				}
			}
		}
		
		if(blockBySpace && !onGround(e, dx, dy)) {
			return false;
		}
		
		return canMove;
	}
	
	public boolean onGround(Entity e) {
		return onGround(e,0,0);
	}
	public boolean onGround(Entity e, float dx, float dy) {
		boolean onGround = false;
		
		TiledMapTileLayer currentLayer = (TiledMapTileLayer)tiledMap.getLayers().get(getCurrentMapLayerIndex(e));
		
		Cell currentCell;
		
		for (Vector2 v : e.getLstCollisions()) {
			currentCell = currentLayer.getCell((int)(((v.x+dx))/32), (int)(((v.y+dy))/32));
			
			if(currentCell != null && currentCell.getTile() != null && "passable".equals(currentCell.getTile().getProperties().get("collision"))){
				return true;
			}
			
			for (GameMapCollisionRectangle r : lstPassableCollision) {
				if(r.getLevel() == e.getLevel() && !r.isDestroy() && r.contains(v,dx,dy)) {
					return true;
				}
			}
		}
		
		return onGround;
	}
	
	
	public boolean onWall(Entity e) {
		boolean onGround = false;
		
		TiledMapTileLayer currentLayer = (TiledMapTileLayer)tiledMap.getLayers().get(getCurrentMapLayerIndex(e));
		
		Cell currentCell;
		
		for (Vector2 v : e.getLstCollisions()) {
			currentCell = currentLayer.getCell((int)((v.x)/32), (int)((v.y)/32));
			
			if(currentCell != null && currentCell.getTile() != null && "wall".equals(currentCell.getTile().getProperties().get("collision"))){
				return true;
			}
		}
		
		return onGround;
	}
	
	public void renderLayer0() {
		tiledMapRenderer.render(layers0);
	}
	public void renderLayer1() {
		tiledMapRenderer.render(layers1);
	}
	public void renderLayer2() {
		tiledMapRenderer.render(layers2);
	}

	public int getCurrentMapLayerIndex(Entity e) {
		int index = 0;
		if(e.getLevel() == 0) {
			index=2;
		}else if(e.getLevel() == 1) {
			index=5;
		}else if(e.getLevel() == 2) {
			index=8;
		}
		
		return index;
	}
	public int getCurrentMapLayerWallIndex(Entity e) {
		int index = 0;
		if(e.getLevel() == 0) {
			index=0;
		}else if(e.getLevel() == 1) {
			index=2;
		}else if(e.getLevel() == 2) {
			index=6;
		}
		
		return index;
	}


	public List<GameMapCollisionRectangle> getLstBlockCollision() {
		return lstBlockCollision;
	}


	public List<GameMapCollisionRectangle> getLstPassableCollision() {
		return lstPassableCollision;
	}
}
