package fr.jeremylebrun.ld41.map;

import com.badlogic.gdx.graphics.Texture;

import fr.jeremylebrun.ld41.core.Ressources;
import fr.jeremylebrun.ld41.core.Shared;
import fr.jeremylebrun.ld41.entities.EnemiFlesh;
import fr.jeremylebrun.ld41.entities.Entity;
import fr.jeremylebrun.ld41.entities.PlayerBullet;
import fr.jeremylebrun.ld41.menus.GameMenu;

public class GameMapItem extends Entity {

	private MapItem item;
	private int hit = 0;
	private int life = 2;

	private GameMapCollisionRectangle blockRectangle;
	private GameMapCollisionRectangle passableRectangle;
	
	private char letter;
	private boolean giveDollars;

	public GameMapItem(GameMenu game, MapItem item, float x, float y, float width, float height, int level) {
		super(x, y, width, height, level);
		this.item = item;
		this.giveDollars = false;
		
		if(item == MapItem.BOX1 || item == MapItem.BOX2) {
			blockRectangle = new GameMapCollisionRectangle(this, x, y, width, height, level, "block");
			passableRectangle = new GameMapCollisionRectangle(this, x, y, width, height, level+1, "passable");

			game.getMap().getLstBlockCollision().add(blockRectangle);
			game.getMap().getLstPassableCollision().add(passableRectangle);
		}
	}
	public GameMapItem(GameMenu game, MapItem item, float x, float y, float width, float height, int level, char letter) {
		super(x, y, width, height, level);
		this.item = item;
		this.letter = letter;
	}
	public GameMapItem(GameMenu game, MapItem item, float x, float y, float width, float height, int level, boolean giveDollars) {
		super(x, y, width, height, level);
		this.item = item;
		this.letter = letter;
		this.giveDollars = giveDollars;
	}

	/**
	 * Dessine l'entite
	 * @param game
	 */
	public void render(GameMenu game) {
		if(!destroy) {
			if(hit>0) {
				Shared.GAME_BATCH.setColor(1f-(hit*0.3f), 1f-(hit*0.3f), 1f-(hit*0.3f), 1f);
			}
			Shared.GAME_BATCH.draw(getTexture(), (int)(x-(width/2)),(int)(y-(height/2)));
			Shared.GAME_BATCH.setColor(1f,1f,1f, 1f);
		}

	}


	/**
	 * Dessine l'entite
	 * @param game
	 */
	public void update(GameMenu game) {
		if(!destroy) {
			if(item == MapItem.GUN1) {
				if(game.getPlayer().onContact(this)) {
					Ressources.SOUND_PICKUP.play();
					game.getPlayer().setGunLevel(1);
					destroy();
				}
			}else if(item == MapItem.MEDKIT) {
					if(game.getPlayer().onContact(this)) {
						Ressources.SOUND_PICKUP.play();
						game.getPlayer().heal(game);
						destroy();
					}
			}else if(item == MapItem.HEART) {
				if(game.getPlayer().onContact(this)) {
					Ressources.SOUND_PICKUP.play();
					game.getPlayer().addLife(game);
					destroy();
				}
			}else if(item == MapItem.LETTER) {
				if(game.getPlayer().onContact(this)) {
					Ressources.SOUND_PICKUP.play();
					game.getHud().addLetter(letter);
					destroy();
				}
			}else if(item == MapItem.DOLLARS) {
					if(game.getPlayer().onContact(this)) {
						Ressources.SOUND_PICKUP.play();
						game.getHud().incDollars();
						destroy();
					}
			}else if(item == MapItem.BOX1 || item == MapItem.BOX2) {
				for (Entity e : game.getLayerEntities(level)) {
					PlayerBullet pb;
					if(e instanceof PlayerBullet && this.onContact(e)) {
						pb = (PlayerBullet) e;
						this.hit++;
						if(hit > life) {
							destroy();
							if(giveDollars) {
								game.getTmpLayerEntities(level).add(new GameMapItem(game, MapItem.DOLLARS, x, y, 32, 32, level));
							}
							game.getTmpLayerEntities(level).add(new EnemiFlesh(x, y, level, Ressources.TEXTURE_ZOMBI_FLESH[3][0], true,false));
							game.getTmpLayerEntities(level).add(new EnemiFlesh(x, y, level, Ressources.TEXTURE_ZOMBI_FLESH[3][1], true,false));
							game.getTmpLayerEntities(level).add(new EnemiFlesh(x, y, level, Ressources.TEXTURE_ZOMBI_FLESH[3][2], true,false));
							game.getTmpLayerEntities(level).add(new EnemiFlesh(x, y, level, Ressources.TEXTURE_ZOMBI_FLESH[3][3], true,false));
							Ressources.SOUND_SHOOT_EXPLODE_BOX.play();
						}else {
							Ressources.SOUND_SHOOT_IMPACT.play();
						}
						pb.destroyByContact(game);
					}
				}
			}
		}
	}


	public void contactWithBullet() {
		if(item == MapItem.BOX1 || item == MapItem.BOX2) {
			this.hit++;
			if(hit > life) {
				destroy();
			}
		}
	}

	private void destroy() {
		this.destroy = true;
		if(blockRectangle != null && !blockRectangle.isDestroy()) {
			this.blockRectangle.destroyCollision();
		}
		if(blockRectangle != null && !blockRectangle.isDestroy()) {
			this.passableRectangle.destroyCollision();
		}

	}

	private Texture getTexture() {
		if(item == MapItem.GUN1) {
			return Ressources.TX_ITEM_GUN1;
		}else if(item == MapItem.BOX1) {
			return Ressources.TX_ITEM_BOX1;
		}else if(item == MapItem.MEDKIT) {
			return Ressources.TX_ITEM_MEDKIT;
		}else if(item == MapItem.DOLLARS) {
			return Ressources.TX_ITEM_DOLLARS;
		}else if(item == MapItem.HEART) {
			return Ressources.TX_ITEM_HEART;
		}else if(item == MapItem.BOX2) {
			return Ressources.TX_ITEM_BOX2;
		}else if(item == MapItem.LETTER) {
			switch(letter) {
			case 'B' :
				return Ressources.TX_ITEM_LETTER_B;
			case 'i' :
				return Ressources.TX_ITEM_LETTER_i;
			case 'g' :
				return Ressources.TX_ITEM_LETTER_g;
			case 'b' :
				return Ressources.TX_ITEM_LETTER_b2;
			case 'o' :
				return Ressources.TX_ITEM_LETTER_o;
			}
		}
		return null;
	}

}
